@@ -1,5 +1,11 @@
 
 """
+glashammer.application
+~~~~~~~~~~~~~~~~~~~~~~
+
+:copyright: 2008 Glashammer Developers
+:license: MIT
+
 The Glashammer Application.
 """
 
@@ -23,7 +29,50 @@ def default_setup_func(app):
 
 
 class GlashammerApplication(object):
-    """WSGI Application"""
+    """WSGI Application
+
+    This should usually be made using the :func:`make_app` function, which
+    additionally wraps the application in threadlocal session cleaning code.
+
+    `setup_func`
+        The callable to be called to initialize the application
+
+    `instance_dir`
+        The directory to use for instance-specific information
+
+    `config_name`
+        The name of the configuration file in the instance directory
+
+    `config_factory`
+        The dict-like class that will be used for configuration. This is
+        provided in situations where a file-based configuration file is not
+        suitable, such as AppEngine.
+
+    `url_map`
+        A werkzeug.routing.Map instance. Additional application rules will
+        be added to this. If this is not provided a new Map will be created
+        by default.
+
+    `view_map`
+        A map of endpoints to view callables. Views added during setup will
+        be added to this map. If it is not provided, a new dict is created.
+
+    `template_searchpaths`
+        A list of paths to search for templates. Additional paths added
+        during application setup will be added to this list. If none is
+        provided a new list will be created (default).
+
+    `template_filters`
+        A mapping of name to callable which are template filters. Additional
+        filters will be added to this map before the template environment is
+        created. If none is provided a new dict is created (default).
+
+    `template_globals`
+        A mapping of name to variable which will be available in the
+        template's global namespace. Additional variables added during
+        application setup will be added to this map. If this is not provided
+        a new dict will be created.
+    """
 
     default_setup = default_setup_func
 
@@ -32,6 +81,7 @@ class GlashammerApplication(object):
         config_name='config.ini', config_factory=Configuration,
         url_map=None, view_map=None,
         template_searchpaths=None, template_filters=None, template_globals=None,
+        template_tests=None,
         ):
         # just for playing in the shell
         local.application = self
@@ -82,6 +132,11 @@ class GlashammerApplication(object):
             self._template_filters = template_filters
         else:
             self._template_filters = {}
+        
+        if template_tests:
+            self._template_tests = template_tests
+        else:
+            self._template_tests = {}
 
         self._layout_template = None
 
@@ -134,6 +189,7 @@ class GlashammerApplication(object):
             searchpaths=self._template_searchpaths,
             globals=self._template_globals,
             filters=self._template_filters,
+            tests=self._template_tests,
         )
 
         del self._template_searchpaths
@@ -339,6 +395,14 @@ class GlashammerApplication(object):
         self._ensure_not_finalized()
 
         self._template_filters[name] = f
+    
+    def add_template_test(self, name, f):
+        """
+        Add a template filter.
+        """
+        self._ensure_not_finalized()
+        
+        self._template_tests[name] = f
 
     def connect_event(self, event, callback, position='after'):
         """
@@ -389,13 +453,19 @@ class GlashammerApplication(object):
 def make_app(setup_func, instance_dir=None, **kw):
     """Create an application instance.
 
-    `setup_func`   The callable used by the application to set itself up. This
-                   is the main entry point into the application.
+    `setup_func`
+        The callable used by the application to set itself up. This
+        is the main entry point into the application.
 
-    `instance_dir` The directory where instance-related files will be stored.
+    `instance_dir`
+        The directory to use for instance-specific information
+
+    `kw`
+        See :class:`GlashammerApplication`
     """
     application = local('application')
-    application = GlashammerApplication(setup_func, instance_dir)
+    application = GlashammerApplication(setup_func,
+        instance_dir, **kw)
     application = local_manager.make_middleware(application)
     return application
 
